import java.util.Vector;

public class Joueur {

	private String nom;
	public boolean humain;
	private Deck deck;
	private Main main;
	public int score=0;
	
	////////////////////////////////////////////////////////
	//						Constructeur				  //
	////////////////////////////////////////////////////////
	
	public Joueur(String nom){
		this.nom = nom;
	}
	
	////////////////////////////////////////////////////////
	//						Accesseurs					  //
	////////////////////////////////////////////////////////
	

	public int getScore(){
		return this.score;
	}
	public String getNom(){
		return this.nom;
	}
	public Deck getDeck(){
		return this.deck;
	}
	public Main getMain(){
		return this.main;
	}
	public void setDeck(Deck deck){
		this.deck = deck;
	}
	public void setMain (Main main){
		this.main=main;
	}
	
	/**
	 * Calcule le score d'un joueur
	 */
	public void calculScore(){
		this.score+=this.getDeck().km;
		if (this.getDeck().km>=1000){
			this.score+=500;
		}
	}

	////////////////////////////////////////////////////////
	//						Methodes					  //
	////////////////////////////////////////////////////////

	/**
	 * Methode qui affiche la main du joueur si il est humain et qui le fait piocher
	 */
	public void jouerTour(){
		// La methode de tour de jeu pour un joueur humain est simple,
		// il suffit de piocher, puis c'est au joueur de cliquer sur la carte
		// qu'il veut jouer. L'action  alors lieu dans la classe Main
		// (puisque c'est sur la mainqu'il clique).
		Jeu.fenetrePrincipale.afficherMain(this);
		this.piocher();
	}
	
	/**
	 * Methode qui fait piocher le joueur IA et qui lui fait jouer la meilleure carte
	 * 
	 */
	public void jouerTourIA(){
		// La methode du tour de jeu de l'IA est assez complexe.
		// En effet, c'est ici qu'est gre une partie de l'IA du jeu.
		this.piocher();
		int[] indices = this.calculIndices();
		int indiceMax=0;
		int indexDeLIndiceMax=0;
		for (int i=0; i<7; i++){
			if (indices[i]>indiceMax){
				indiceMax = indices[i];
				indexDeLIndiceMax = i;
			}
		}
		Carte c = this.getMain().getCartesDeLaMain()[indexDeLIndiceMax];
		Vector<Joueur> joueursAttaquables = c.joueursAttaquables(this);
		switch(indiceMax){
		case 9: case 8: case 7: case 6: 
			this.getMain().jouerCarte(c,Jeu.partie.joueurEnCours); break;
		
		case 5: this.getMain().jouerCarte(c, Jeu.partie.quiALePlusDeKm()); break;
		
		case 4: case 3: case 2:
			this.getMain().jouerCarte(c, joueursAttaquables.elementAt((int)Math.random()*joueursAttaquables.size()));break;
		
		case 1: this.getMain().jouerCarte(c,Jeu.partie.joueurEnCours); break;
		
		default: if(c.botte){this.getMain().jouerCarte(c,Jeu.partie.joueurEnCours);}
					else{Jeu.partie.pioche.defausser(c);}
						break;
		}
	}
		
	/**
	 * Fin d'un tour, condition de victoire et retrait de surbrillance
	 */
	public void finirTour(){
		this.getDeck().retirerLaSurbrillance();
		// Une des conditions de victoire est place ici.
		if (this.getDeck().km>=1000){
			Jeu.partie.joueurEnCours=null;
			Jeu.partie.finirPartie();
		}else{
			// Si la partie n'est pas gagne, on apelle la classe qui gre
			// le changement de joueur avec pour paramtre le joueur suivant.
			Jeu.partie.joueurEnCours=null;
			int indexDuJoueurEnCours = Jeu.partie.joueurs.indexOf(this);
			Jeu.partie.chgmtDeJoueur= new ChangementDeJoueur((Jeu.partie.joueurs.elementAt((indexDuJoueurEnCours+1)%Jeu.partie.nbJoueurs)));
		}
	}

	/**
	 * Methode complexe qui met des indices d'interet  chaque carte de la main du joueur
	 * @return int[]
	 */
	private int[] calculIndices(){
	/*
	 *	L'IA examine chaque carte de sa main,
	 *	Il fixe un indice "d'intert"  chacune:
	 *		-La carte n'est pas possible  jouer, elle prend l'indice 0
	 *		-La carte est possible  jouer:
	 *			-La carte est necessaire au joueur pour avancer
	 *			 	-fin de limitation, elle prend l'indice 9
	 *				-parade ou feu vert, elle prend l'indice 8
	 *			-La carte peut faire avancer le joueur
	 *				-grosse borne, elle prend l'indice 7
	 *			 	-petite borne, elle prend l'indice 6
	 *			-La carte est une attaque
	 *				-si elle peut tre joue au joueur qui domine,
	 *					elle prend l'indice 5 quelle qu'elle soit.
	 *				-sinon
	 *					-si c'est une attaque de type panne accident ou crevaison
	 *						elle prend l'indice 4
	 *					-si c'est une attaque de type feu rouge, elle prend l'indice 3
	 *					-si c'est une attaque de type limitation, elle prend l'indice 2
	 *			-Si la carte est une botte
	 *				-si elle permettrait au joueur d'avancer, elle prend l'indice 1
	 *				-sinon,
	 *					-si la partie n'est pas trop avance
	 *						(plus de 15 cartes dans la pioche, et si le joueur
	 *						dominant  moins de 750km) elle prend l'indice 0
	 *					si la partie est presque finie, elle prend l'indice 9		
	 */
	
		// La suite n'est qu'une succession de if et de else sans grand interet :)
		// l'important c'est que ca marche !
		int[] indices = new int[7];
		for(int i = 0; i<7; i++){
			Carte c= this.getMain().getCartesDeLaMain()[i];
			Vector<Joueur> joueursAttaquables = c.joueursAttaquables(this);
			if ((c.finLimit&& this.cartePossible(c))
					||(c.botte && (Jeu.partie.quiALePlusDeKm().getDeck().km>750
							|| Jeu.partie.pioche.size()<15))){
				indices[i]=9;
			}else{
				if((c.parade||c.feuV) && this.cartePossible(c)){
					indices[i]=8;
				}else{
					if(c.grosseBorne && this.cartePossible(c)){
						indices[i]=7;
					}else{
						if(c.petiteBorne && this.cartePossible(c)){
							indices[i]=6;
						}else{
							if((c.attaque || c.feuR || c.limit) 
									&& Jeu.partie.quiALePlusDeKm().cartePossible(c)
									&& Jeu.partie.quiALePlusDeKm()!=this){
								indices[i] = 5;
							}else{
								if(c.attaque && joueursAttaquables.size()!=0){
									indices[i] = 4;
								}else{
									if (c.feuR && joueursAttaquables.size()!=0){
										indices[i] = 3;
									}else{
										if(c.limit && joueursAttaquables.size()!=0){
											indices[i] = 2;
										}else{
											if(c.botte && c.getComplementaire().equals(this.getDeck().carteVisibleFeux)){
												indices[i]=1;
											}else{
												indices[i]=0;
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return indices;
	}
	
	/**
	 * Recherche si la main contient une carte de nom c
	 * @param c String
	 * @return boolean
	 */
	public boolean mainContient(String c){
		// Cette methode fait le tour de la main pour savoir si 
		// la carte dont le nom est entr en paramtre est prsente.
		boolean aRetourner = false;
		for (int i=0;i<7;i++){
			if(this.getMain().getCartesDeLaMain()[i].getNom().equals(c)
					&& !aRetourner){
				aRetourner=true;
			}
		}
		return aRetourner;
	}
	
	/**
	 * Methode qui joue un coup fourr, fait piocher, ajoute les points au joueur, pose sa borne etc etc
	 * 
	 * @param c Carte
	 */
	public void joueUnCoupFoure(Carte c){
		// On change de joueur en cours,
		// on augmente le score,
		// on fait piocher le joueur qui va poser une borne,
		this.score+=500;
		this.piocher();
		String botte="";
		switch(c.getNom().charAt(0)){
		case 'A': botte="As du volant"; break;
		case 'P': botte="Citerne"; break;
		case 'C': botte="Increvable"; break;
		default: botte="Vhicule prioritaire"; break;
		}
		for(int i=0; i<6; i++){
			// On va chercher la botte en question dans la main
			// et on la joue.
			if (this.getMain().getCartesDeLaMain()[i].getNom().equals(botte)){
				Carte laBotte = this.getMain().getCartesDeLaMain()[i];
				this.getMain().retirerCarte(laBotte);
				this.getDeck().ajouterCarte(laBotte);
				Jeu.fenetrePrincipale.pack();
				Jeu.fenetrePrincipale.repaint();
				Jeu.partie.joueurEnCours=this;
				// Ensuite, le joueur qui a jou le coup fourr commence un tour,
				// il "coupe" la partie.
				Jeu.partie.joueurEnCours.getDeck().retirerLaSurbrillance();
				Jeu.partie.joueurEnCours=this;
				Jeu.partie.chgmtDeJoueur = new ChangementDeJoueur(this);
			}
		}
	}
	
	/**
	 * Pioche une carte et actualise la pioche
	 */
	private void piocher(){
		// Lorsqu'un joueur pioche, le paquet est dcrement, et si le joueur
		// est humain, la carte est affiche sur la pioche.
		this.getMain().getCartesDeLaMain()[6] = Jeu.partie.pioche.lastElement();
		Jeu.partie.pioche.removeElement(Jeu.partie.pioche.lastElement());
		if(this.humain){
			this.getMain().getCartesDeLaMain()[6].afficherCarteDans(Jeu.partie.pioche.getPanelDeLaPioche());
		}
		Jeu.partie.pioche.calculerCartesPiocheRestantes();
		Jeu.partie.pioche.getPanelDeLaPioche().repaint();
		Jeu.fenetrePrincipale.pack();
		Jeu.fenetrePrincipale.repaint();
	}

	/**
	 * Mthode qui regroupe en un seul booleen le fait qu'une carte soit possible sur le deck de this.
	 * @param c Carte
	 * @return boolean
	 */
	public boolean cartePossible(Carte c){
		return ((c.petiteBorne && this.petiteBornePossible()
					    && Integer.parseInt(c.getNom())+this.getDeck().km <= 1000)
				||(c.grosseBorne && this.grosseBornePossible()
						&& Integer.parseInt(c.getNom())+this.getDeck().km <= 1000)
				||(c.finLimit && this.finDeLimitPossible())
				||(c.limit && this.limitPossible())
				||(c.feuR && this.feuRougePossible())
				||(c.feuV && this.feuVertPossible())
				||(c.accident && this.accidentPossible())
				||(c.crevaison && this.crevaisonPossible())
				||(c.panne && this.pannePossible())
				||(c.essence && this.essencePossible())
				||(c.reparation && this.reparationPossible())
				||(c.roueDeSecours && this.roueDeSecoursPossible())
				||c.botte);		
	}
	
	
	////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////
	
	// Ces methodes prcisent si chaque type de carte peut tre au joue
	// au joueur this.
	
	/**
	 * Si une grosse borne peut tre pose
	 * @return boolean
	 */
	private boolean grosseBornePossible(){
		return((this.getDeck().prioritaire() 
				&& !this.getDeck().carteVisibleFeux.attaque)
				|| (this.getDeck().carteVisibleFeux.feuV
				&& !this.getDeck().carteVisibleLimitations.limit));
	}	
	
	/**
	 * Si une petite borne peut tre pose
	 * @return boolean
	 */
	private boolean petiteBornePossible(){
		return ((this.getDeck().prioritaire() 
				&& !this.getDeck().carteVisibleFeux.attaque)
				|| this.getDeck().carteVisibleFeux.feuV);
	}
	
	/**
	 * Si une fin de limitation peut tre pose
	 *@return boolean
	 */
	private boolean finDeLimitPossible(){
		return this.getDeck().carteVisibleLimitations.limit;
	}
	
	/**
	 * Si une limitation peut tre pose
	 *@return boolean
	 */
	private boolean limitPossible(){
		return (!this.getDeck().carteVisibleLimitations.limit 
		&& !this.getDeck().prioritaire());
	}
	
	/**
	 * Si un feu vert peut tre pos
	 * @return boolean
	 */
	private boolean feuVertPossible(){
		return (!this.getDeck().carteVisibleFeux.feuV
				&& !this.getDeck().carteVisibleFeux.attaque
				&& !this.getDeck().prioritaire());
	}
	
	/**
	 * Si un feu rouge peut tre pos
	 * @return boolean
	 */
	private boolean feuRougePossible(){
		return(this.getDeck().carteVisibleFeux.feuV
				&& !this.getDeck().prioritaire());
	}
	
	/**
	 * Si une panne d'essence peut tre pose
	 * @return boolean
	 */
	private boolean pannePossible(){
		return ((this.getDeck().carteVisibleFeux.feuV
				||(this.getDeck().prioritaire()
						&& !this.getDeck().carteVisibleFeux.attaque))
				&& !this.getDeck().citerne());
	}
	
	/**
	 * Si un accident peut tre pose
	 * @return boolean
	 */
	private boolean accidentPossible(){
		return ((this.getDeck().carteVisibleFeux.feuV
				||(this.getDeck().prioritaire()
						&& !this.getDeck().carteVisibleFeux.attaque))
				&& !this.getDeck().as());
	}
	
	/**
	 * Si une crevaison peut tre pose
	 * @return boolean
	 */
	private boolean crevaisonPossible(){
		return ((this.getDeck().carteVisibleFeux.feuV
				||(this.getDeck().prioritaire()
						&& !this.getDeck().carteVisibleFeux.attaque))
				&& !this.getDeck().increvable());
	}
	
	/**
	 * Si un plein d'essence peut tre pos
	 * @return boolean
	 */
	private boolean essencePossible(){
		return this.getDeck().carteVisibleFeux.panne;
	}
	
	/**
	 * Si une rparation peut tre pose
	 * @return boolean
	 */
	private boolean reparationPossible(){
		return this.getDeck().carteVisibleFeux.accident;
	}
	
	/**
	 * Si une roue de secours peut tre pose
	 * @return boolean
	 */
	private boolean roueDeSecoursPossible(){
		return this.getDeck().carteVisibleFeux.crevaison;
	}
	
	////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////
	
}
